Stan Math Library: stan/math/prim/scal/prob/neg_binomial_2_log_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

neg_binomial_2_log_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_2_LOG_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_2_LOG_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_less.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/binomial_coefficient_log.hpp>
13 #include <stan/math/prim/scal/fun/multiply_log.hpp>
14 #include <stan/math/prim/scal/fun/digamma.hpp>
15 #include <stan/math/prim/scal/fun/lgamma.hpp>
16 #include <stan/math/prim/scal/fun/log_sum_exp.hpp>
17 #include <stan/math/prim/scal/fun/value_of.hpp>
18 #include <stan/math/prim/scal/meta/include_summand.hpp>
19 #include <stan/math/prim/scal/fun/grad_reg_inc_beta.hpp>
20 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
21 #include <boost/math/special_functions/digamma.hpp>
22 #include <boost/random/negative_binomial_distribution.hpp>
23 #include <boost/random/variate_generator.hpp>
24 #include <cmath>
25
26 namespace stan {
27
28 namespace math {
29
30 // NegBinomial(n|eta, phi) [phi > 0; n >= 0]
31 template <bool propto,
32 typename T_n,
33 typename T_log_location, typename T_precision>
34 typename return_type<T_log_location, T_precision>::type
35 neg_binomial_2_log_log(const T_n& n,
36 const T_log_location& eta,
37 const T_precision& phi) {
38 typedef typename stan::partials_return_type<T_n, T_log_location,
39 T_precision>::type
40 T_partials_return;
41
42 static const char* function("stan::prob::neg_binomial_2_log_log");
43
44 using stan::math::check_finite;
45 using stan::math::check_nonnegative;
46 using stan::math::check_positive_finite;
47 using stan::math::value_of;
48 using stan::math::check_consistent_sizes;
49 using stan::math::include_summand;
50
51 // check if any vectors are zero length
52 if (!(stan::length(n)
53 && stan::length(eta)
54 && stan::length(phi)))
55 return 0.0;
56
57 T_partials_return logp(0.0);
58 check_nonnegative(function, "Failures variable", n);
59 check_finite(function, "Log location parameter", eta);
60 check_positive_finite(function, "Precision parameter", phi);
61 check_consistent_sizes(function,
62 "Failures variable", n,
63 "Log location parameter", eta,
64 "Precision parameter", phi);
65
66 // check if no variables are involved and prop-to
67 if (!include_summand<propto, T_log_location, T_precision>::value)
68 return 0.0;
69
70 using stan::math::multiply_log;
71 using stan::math::log_sum_exp;
72 using stan::math::digamma;
73 using stan::math::lgamma;
74 using std::exp;
75 using std::log;
76
77 // set up template expressions wrapping scalars into vector views
78 VectorView<const T_n> n_vec(n);
79 VectorView<const T_log_location> eta_vec(eta);
80 VectorView<const T_precision> phi_vec(phi);
81 size_t size = max_size(n, eta, phi);
82
83 OperandsAndPartials<T_log_location, T_precision>
84 operands_and_partials(eta, phi);
85
86 size_t len_ep = max_size(eta, phi);
87 size_t len_np = max_size(n, phi);
88
89 VectorBuilder<true, T_partials_return, T_log_location> eta__(length(eta));
90 for (size_t i = 0, size = length(eta); i < size; ++i)
91 eta__[i] = value_of(eta_vec[i]);
92
93 VectorBuilder<true, T_partials_return, T_precision> phi__(length(phi));
94 for (size_t i = 0, size = length(phi); i < size; ++i)
95 phi__[i] = value_of(phi_vec[i]);
96
97
98 VectorBuilder<true, T_partials_return, T_precision>
99 log_phi(length(phi));
100 for (size_t i = 0, size = length(phi); i < size; ++i)
101 log_phi[i] = log(phi__[i]);
102
103 VectorBuilder<true, T_partials_return, T_log_location, T_precision>
104 logsumexp_eta_logphi(len_ep);
105 for (size_t i = 0; i < len_ep; ++i)
106 logsumexp_eta_logphi[i] = log_sum_exp(eta__[i], log_phi[i]);
107
108 VectorBuilder<true, T_partials_return, T_n, T_precision>
109 n_plus_phi(len_np);
110 for (size_t i = 0; i < len_np; ++i)
111 n_plus_phi[i] = n_vec[i] + phi__[i];
112
113 for (size_t i = 0; i < size; i++) {
114 if (include_summand<propto>::value)
115 logp -= lgamma(n_vec[i] + 1.0);
116 if (include_summand<propto, T_precision>::value)
117 logp += multiply_log(phi__[i], phi__[i]) - lgamma(phi__[i]);
118 if (include_summand<propto, T_log_location, T_precision>::value)
119 logp -= (n_plus_phi[i])*logsumexp_eta_logphi[i];
120 if (include_summand<propto, T_log_location>::value)
121 logp += n_vec[i]*eta__[i];
122 if (include_summand<propto, T_precision>::value)
123 logp += lgamma(n_plus_phi[i]);
124
125 if (!is_constant_struct<T_log_location>::value)
126 operands_and_partials.d_x1[i]
127 += n_vec[i] - n_plus_phi[i]
128 / (phi__[i]/exp(eta__[i]) + 1.0);
129 if (!is_constant_struct<T_precision>::value)
130 operands_and_partials.d_x2[i]
131 += 1.0 - n_plus_phi[i]/(exp(eta__[i]) + phi__[i])
132 + log_phi[i] - logsumexp_eta_logphi[i] - digamma(phi__[i])
133 + digamma(n_plus_phi[i]);
134 }
135 return operands_and_partials.value(logp);
136 }
137
138 template <typename T_n,
139 typename T_log_location, typename T_precision>
140 inline
141 typename return_type<T_log_location, T_precision>::type
142 neg_binomial_2_log_log(const T_n& n,
143 const T_log_location& eta,
144 const T_precision& phi) {
145 return neg_binomial_2_log_log<false>(n, eta, phi);
146 }
147 }
148 }
149 #endif
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
lgamma.hpp
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
digamma.hpp
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
stan::math::neg_binomial_2_log_log
return_type< T_log_location, T_precision >::type neg_binomial_2_log_log(const T_n &n, const T_log_location &eta, const T_precision &phi)
Definition: neg_binomial_2_log_log.hpp:35
binomial_coefficient_log.hpp
is_constant_struct.hpp
stan::math::log_sum_exp
fvar< T > log_sum_exp(const std::vector< fvar< T > > &v)
Definition: log_sum_exp.hpp:14
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
log_sum_exp.hpp
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::exp
fvar< T > exp(const fvar< T > &x)
Definition: exp.hpp:10
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
grad_reg_inc_beta.hpp
stan::math::check_finite
bool check_finite(const char *function, const char *name, const T_y &y)
Return true if y is finite.
Definition: check_finite.hpp:62
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_less.hpp
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16

[ Stan Home Page ] © 2011–2016, Stan Development Team.